/*
 * Copyright (c) 2013, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/**
 * @test
 * @bug 8029955
 * @summary test that the size of whitespace lookup buffer is adjusted as needed
 * @run main EntityScannerTest
 */

import java.io.*;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;


/**
 *
 * @author Joe Wang huizhe.wang@oracle.com
 */
public class EntityScannerTest {

    /**
     * main method.
     *
     * @param args Standard args.
     */
    public static void main(String[] args) {
        try {
            StringBuilder builder = new StringBuilder();
            builder.append("<root attr=\"");
            for (int i = 0; i < 200; i++) {
                builder.append("\n");
            }
            builder.append("foo.");
            builder.append("\" />");
            final XMLReader reader = XMLReaderFactory.createXMLReader();
            System.out.println(reader.getClass().getName());
            reader.parse(new InputSource(new StringReader(builder.toString())));
        } catch (ArrayIndexOutOfBoundsException e) {
            throw new RuntimeException("Test failed: ArrayIndexOutOfBoundsException " + e.getMessage());
        } catch (Exception e) {
            throw new RuntimeException(e.getMessage());
        }
    }

}
