/*
 * Copyright (c) 2002, 2012, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/*
 * @test
 * @bug 4529320
 * @compile -XDignore.symbol.file UnresolvedExceptions.java
 * @run main UnresolvedExceptions
 * @summary Verifying jvm won't segv if exception not available
 * @author Joseph D. Darcy, ksrini
 */

import java.io.File;
import java.util.ArrayList;
import java.util.List;


public class UnresolvedExceptions extends TestHelper {

    public static void main(String... args) throws Exception {
        final String fname = "Foo";
        List<String> buffer = new ArrayList<>();
        buffer.add("public class " + fname + " {");
        buffer.add("    public static void main(String[] argv) throws "
                       + "Foo.SomeException {");
        buffer.add("        System.exit(0);");
        buffer.add("    }");
        buffer.add("    static class SomeException extends RuntimeException{}");
        buffer.add("}");

        File testJavaFile = new File("Foo" + JAVA_FILE_EXT);
        createFile(testJavaFile, buffer);
        compile(testJavaFile.getName());
        TestResult tr = doExec(javaCmd, "-cp", ".", fname);
        if (!tr.isOK()) {
            System.out.println(tr);
            throw new RuntimeException("java -cp ... failed");
    }
}
}
